let time = ref 0

let pi = 4. *. atan 1.
let circle_point (x, y) r n_max idx =
  let theta = 2. *. pi *. (float idx) /. (float n_max) in
  (x +. r *. cos theta, y +. r *. sin theta)

let circle_points (x, y) = Array.init 31 (circle_point (x, y) 0.5 31)

let orig = (0., 0.)
let circle r theta = (r *. cos theta, r *. sin theta)
let cutter_points theta y = [|
  (0., y);
  (0.5 *. cos theta, 0.5 *. sin theta +. y);
  (0.45 *. cos (theta -. 0.1 *. pi), 0.45 *. sin (theta -. 0.1 *. pi) +. y)
|]

(* stable condition *)
let a_s = 0.02
let omega_1s = 2. *. pi

(* chatter condition *)
let a_c = 0.2
let omega_1c = 2.7 *. pi

let b = 0.01

let omega_1 = 2. *. pi
let omega_2 = 2. *. pi
let vf = 0.05 (* 送り速度 *)


(* let y_vib t = a_s *. cos (omega_1s *. t) *)
let y_vib t = a_c *. cos (omega_1c *. t)
let y_wk t = b *. cos (omega_2 *. t)

let () =
  ignore (Glut.init Sys.argv);
  Glut.initDisplayMode ~double_buffer:true ();
  Glut.initWindowSize ~w:512 ~h:512;
  ignore (Glut.createWindow ~title:"LablGL Cutting Demo");
  GlClear.color (0., 0., 0.);
  GlDraw.shade_model `smooth;
  let render () =
    GlMat.mode `projection;
    GlMat.load_identity ();
    GluMat.ortho2d ~x:(-1.5, 1.5) ~y:(-1.5, 1.5);
    GlMat.mode `modelview;
    GlMat.load_identity ();

    GlClear.clear [`color];
    let t = (float !time) /. 200. in
    GlDraw.color (0.8, 0.8, 0.8);
    GlDraw.begins `quads;
    GlDraw.vertex2 (-2., (y_wk t));
    GlDraw.vertex2 (2., (y_wk t));
    GlDraw.vertex2 (2., -1. +. (y_wk t));
    GlDraw.vertex2 (-2., -1. +. (y_wk t));
    GlDraw.ends ();

    GlDraw.color (0., 0., 0.);
    for n = 1 to 40 do
      GlDraw.begins `polygon;
      Array.iter GlDraw.vertex2 (circle_points ((float n) *. vf, y_vib (t -. 0.05 *. (float n)) +. (y_wk t)));
      GlDraw.ends ();
    done;

    GlDraw.color (0.5, 0.5, 0.5);
    GlDraw.begins `polygon;
    Array.iter GlDraw.vertex2 (circle_points (0., y_vib t));
    GlDraw.ends ();

    GlDraw.color (0.6, 0.6, 0.6);
    for n = 0 to 2 do
      GlDraw.begins `triangles;
      Array.iter GlDraw.vertex2 (cutter_points ((float (1+2*n)) *. pi /. 3. +. 2.*.t) (y_vib t));
      GlDraw.ends ();
    done;
    Gl.flush ();
    time := !time + 1;
    Glut.swapBuffers ()
  in
  Glut.displayFunc ~cb:render;
  Glut.idleFunc ~cb:(Some Glut.postRedisplay);
  Glut.keyboardFunc ~cb:(fun ~key ~x ~y -> if key=27 then exit 0);
  Glut.mainLoop ()
