
open Realext
open Complexalgebra

open Lacaml.C

let z = 2

let m_x = 10.
let m_y = 10.

let k0_x = ((2. *. pi) *^ 2) *. 10. ** 7.
let k0_y = ((2. *. pi) *^ 2) *. 10. ** 7.

let k_x = k0_x
let k_y = k0_y

let c_x = 300.
let c_y = 300.

let wn_x = sqrt (k_x /. m_x)
let wn_y = sqrt (k_y /. m_y)

let kt = 1500. *. 10. ** 6.
let kr = 0.3

(* A-matrix *)
let a0 ths the =
  let n_4pi = (float_of_int z) /. (4. *. pi) in
  let (cse, sne) = (cos (2. *. the), sin (2. *. the)) in
  let (css, sns) = (cos (2. *. ths), sin (2. *. ths)) in
  [| [| (n_4pi *. ((-.sne-.2.*.the+.kr*.cse) -. (-.sns-.2.*.ths+.kr*.css))) +< i 0.;
       (n_4pi *. ((-.sne-.2.*.the+.kr*.cse) -. (-.sns-.2.*.ths+.kr*.css))) +< i 0. |];
    [| (n_4pi *. ((-.sne+.2.*.the+.kr*.cse) -. (-.sns+.2.*.ths+.kr*.css))) +< i 0. ;
      (n_4pi *. ((-.sne+.2.*.the+.kr*.cse) -. (-.sns+.2.*.ths+.kr*.css))) +< i 0. |] |]

let we = if wn_x > wn_y then 2. *. wn_x else 2. *. wn_y


let plot_stability_point k w eig =
  let l_re = real (Complex.neg (Complex.inv eig)) in
  let l_im = imag (Complex.neg (Complex.inv eig)) in
  let alim = -. 1000. *. l_re /. kt *. (1. +. ((l_im /. l_re) *^ 2)) in
  let ep = pi -. 2. *. atan (l_im /. l_re) in
  if 0. < alim && alim < 50. then
    begin
      let n_rpm = 60.0 *. w /. ((float_of_int z) *. (2. *. (float_of_int (k-1)) *. pi +. ep)) in
      print_endline ((string_of_float n_rpm) ^ " " ^ (string_of_float alim))
    end

let rec stability_diagram k f0 n =
	let w = 2. *. pi *. ((float_of_int f0) +. (float_of_int n) *. 0.05) in
  let g = [| [| Complex.inv ((k_x -. m_x *. (w *^2)) +< i (c_x *. w)); Complex.zero |];
	    [| Complex.zero ; Complex.inv ((k_y -. m_y *. (w *^2)) +< i (c_y *. w)) |] |] in
  let (_, eig_v, _) = eigen_values (a0 0. (0.25 *. pi) *! g) in
  Array.iter (plot_stability_point k w) (Vec.to_array eig_v);
  (* tail recursion *)
  if 2. *. pi *. ((float_of_int f0) +. (float_of_int (n+1)) *. 0.05) >= we then ()
  else stability_diagram k f0 (n+1)

let () =
	for deg = 1 to 5 do
		stability_diagram deg 980 0
	done
