
let pi = 4. *. atan 1.
let circle_point (x, y) r n_max idx =
	let theta = 2. *. pi *. (float idx) /. (float n_max) in
	(x +. r *. cos theta, y +. r *. sin theta)

let circle_points (x, y) = Array.init 31 (circle_point (x, y) 1. 31)

let ( +! ) (u1,u2,u3) (v1,v2,v3) =
	(u1 +. v1, u2 +. v2, u3 +. v3)

let ( *< ) c (u1, u2, u3) =
	(c *. u1, c *. u2, c *. u3)

let runge_kutta_next (x, v) df h =
	let (kx1, kv1) = df (x,v) in
	let (kx2, kv2) = df ((x +! ((0.5*.h) *< kx1)), (v +! ((0.5*.h) *< kv1))) in
	let (kx3, kv3) = df ((x +! ((0.5*.h) *< kx2)), (v +! ((0.5*.h) *< kv2))) in
	let (kx4, kv4) = df ((x +! (h *< kx3)), (v +! (h *< kv3))) in
	(x +! ((h/.6.) *< (kx1 +! (2. *< kx2) +! (2. *< kx3) +! kx4)), v +! ((h/.6.) *< (kv1 +! (2. *< kv2) +! (2. *< kv3) +! kv4)))

let k = 100.
let m = 1.
let df ((x1,x2,x3), (v1,v2,v3)) =
	((v1,v2,v3), ((k /. m) *. (x2 -. x1), -.(k /. m) *. (x2 -. x1) +. (k /. m) *. (x3 -. x2), -.(k /. m) *. (x3 -. x2)))


let x1 = ref 1.
let v1 = ref 0.

let x2 = ref 0.
let v2 = ref 0.

let x3 = ref 0.
let v3 = ref 0.

let y1 = ref 0.
let y2 = ref 0.
let y3 = ref 0.

let t = ref 0.

let () =
  ignore (Glut.init Sys.argv);
  Glut.initDisplayMode ~double_buffer:true ();
  Glut.initWindowSize ~w:512 ~h:512;
  ignore (Glut.createWindow ~title:"LablGL Triple Demo");
  GlClear.color (0.6, 0.6, 1.0);
  GlDraw.shade_model `smooth;
	
  let render () =
    GlMat.mode `projection;
    GlMat.load_identity ();
    GluMat.ortho2d ~x:(-10., 10.) ~y:(-10., 10.);
    GlMat.mode `modelview;
    GlMat.load_identity ();
    GlClear.clear [ `color ];
		let ((x1_new, x2_new, x3_new), (v1_new, v2_new, v3_new)) = runge_kutta_next ((!x1, !x2, !x3), (!v1, !v2, !v3)) df 0.01 in
		x1 := x1_new; x2 := x2_new; x3 := x3_new;
		v1 := v1_new; v2 := v2_new; v3 := v3_new;
		y1 := 0.; y2 := 0.; y3 := 0.;
		GlDraw.begins `lines;
		GlDraw.vertex2 (!x1-.5., !y1);
		GlDraw.vertex2 (!x2, !y2);
		GlDraw.vertex2 (!x2, !y2);
		GlDraw.vertex2 (!x3+.5., !y3);
		GlDraw.ends ();
		
    GlDraw.begins `polygon;
    Array.iter GlDraw.vertex2 (circle_points (!x1-.5., !y1));
    GlDraw.ends ();
		
		GlDraw.begins `polygon;
    Array.iter GlDraw.vertex2 (circle_points (!x2, !y2));
    GlDraw.ends ();
		
    GlDraw.begins `polygon;
    Array.iter GlDraw.vertex2 (circle_points (!x3+.5., !y3));
    GlDraw.ends (); 
		Gl.flush ();
    Glut.swapBuffers ()
	in
  Glut.displayFunc ~cb:render;
  Glut.idleFunc ~cb:(Some Glut.postRedisplay);
	(* Escキーで終了 *)
  Glut.keyboardFunc ~cb:(fun ~key ~x ~y -> if key=27 then exit 0);
  Glut.mainLoop ()