
Require Import List.

Definition swap (pair : nat*nat) : nat*nat :=
  let (n1,n2) := pair in
  (n2,n1).

Lemma double_swap : forall (pair : nat*nat), swap (swap pair) = pair.
  intros [m n].
  reflexivity.
Qed.

Definition conjugate_diagram (d : list (nat*nat)) : list (nat*nat) := map swap d.

Lemma double_conjugate : forall (d : list (nat*nat)), conjugate_diagram (conjugate_diagram d) = d.
Proof.
  intros.
  induction d; simpl.
  - now reflexivity.
  - rewrite double_swap, IHd.
    now reflexivity.
Qed.

Definition composition (f g : nat*nat -> nat*nat) (lat : nat*nat) : nat*nat :=
  f (g lat).

Lemma map_composition : forall (f g : nat*nat -> nat*nat) l, map f (map g l) = map (composition f g) l.
Proof.
  induction l; simpl.
  - reflexivity.
  - rewrite IHl.
    reflexivity.
Qed.

Lemma map_id : forall f l, (forall x : nat*nat, f x = x) -> map f l = l.
Proof.
  induction l; simpl; intros.
  - now reflexivity.
  - rewrite (IHl H), H.
    now reflexivity.
Qed.

Lemma double_conjugate' : forall (d : list (nat*nat)), conjugate_diagram (conjugate_diagram d) = d.
Proof.
  intros.
  unfold conjugate_diagram.
  rewrite map_composition.
  apply map_id.
  apply double_swap.
Qed.